package com.engine.kq.cmd.attendanceEvent;

import com.alibaba.fastjson.JSONObject;
import com.engine.common.biz.AbstractCommonCommand;
import com.engine.common.entity.BizLogContext;
import com.engine.core.interceptor.CommandContext;
import com.engine.kq.biz.KQGroupMemberComInfo;
import com.engine.kq.biz.KQOvertimeRulesBiz;
import com.engine.kq.biz.KQWorkTime;
import com.engine.kq.entity.WorkTimeEntity;
import com.engine.kq.enums.DurationTypeEnum;
import com.engine.kq.util.KQDurationCalculatorUtil;
import com.engine.kq.util.TimeRangeCalculator;
import org.apache.commons.lang3.StringUtils;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.general.TimeUtil;
import weaver.general.Util;
import weaver.hrm.User;
import weaver.hrm.resource.ResourceComInfo;
import weaver.systeminfo.SystemEnv;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * 加班用的时长计算
 */
public class GetOverTimeWorkDurationCmd extends AbstractCommonCommand<Map<String, Object>> {

    public GetOverTimeWorkDurationCmd(Map<String, Object> params, User user) {
        this.user = user;
        this.params = params;
    }

    @Override
    public Map<String, Object> execute(CommandContext commandContext) {
        Map<String, Object> retmap = new HashMap<String, Object>();
        RecordSet rs = new RecordSet();
        String sql = "";
        try {
            BaseBean bb = new BaseBean();
            String resourceId = Util.null2String(params.get("resourceId"));
            String fromDate = Util.null2String(params.get("fromDate"));
            String toDate = Util.null2String(params.get("toDate"));
            String fromTime = Util.null2String(params.get("fromTime"));
            String toTime = Util.null2String(params.get("toTime"));
            String overtime_type = Util.null2String(params.get("overtime_type"));
            String timestamp = Util.null2String(params.get("timestamp"));
            bb.writeLog("计算时长");
            int minimumUnit = KQOvertimeRulesBiz.getMinimumUnit();
            //
            KQWorkTime kqWorkTime = new KQWorkTime();
            WorkTimeEntity kqWorkTimeEntity = kqWorkTime.getWorkTime(resourceId, fromDate);
            if (kqWorkTimeEntity != null) {
                String kqType = Util.null2String(kqWorkTimeEntity.getKQType());
                if ("3".equalsIgnoreCase(kqType)) {
                    writeLog("自由班制不计算加班");
                    retmap.put("status", "1");
                    retmap.put("message", "" + weaver.systeminfo.SystemEnv.getHtmlLabelName(10005330, weaver.general.ThreadVarLanguage.getLang()) + "");
                    return retmap;
                }
            }

            //休班次的加班时长
            String restSerialValue = "";
            restSerialValue = isRestSerialCalculate(resourceId, kqWorkTime);
            if (StringUtils.isNotEmpty(restSerialValue)) {
                retmap.put("duration", restSerialValue);
                retmap.put("timestamp", timestamp);
                retmap.put("status", "1");
                return retmap;
            }

            ResourceComInfo rci = new ResourceComInfo();
            KQGroupMemberComInfo kqGroupMemberComInfo = new KQGroupMemberComInfo();
            String groupid = kqGroupMemberComInfo.getKQGroupId(resourceId, fromDate);
            bb.writeLog("计算时长groupid：" + groupid);
            if (resourceId.length() > 0 && groupid.length() == 0) {
                retmap.put("status", "-1");
                retmap.put("message", rci.getLastname(resourceId) + "," + fromDate + "" + weaver.systeminfo.SystemEnv.getHtmlLabelName(10005329, weaver.general.ThreadVarLanguage.getLang()) + "");
                return retmap;
            }

            bb.writeLog("计算时长fromDate：" + fromDate + "toDate:" + toDate + "fromTime:" + fromTime + "toTime:" + toTime);
            //加班默认是工作日加班
            KQDurationCalculatorUtil kqDurationCalculatorUtil = new KQDurationCalculatorUtil.DurationParamBuilder(resourceId).
                    fromDateParam(fromDate).toDateParam(toDate).fromTimeParam(fromTime).toTimeParam(toTime).computingModeParam("1").
                    durationRuleParam(minimumUnit + "").durationTypeEnumParam(DurationTypeEnum.OVERTIME).
                    overtime_typeParam(overtime_type).build();

            Map<String, Object> durationMap = kqDurationCalculatorUtil.getWorkDuration();


            //管理人员12小时班制考勤组id
            String glrygroupid = bb.getPropValue("nbkq_main", "glrygroupid");
            bb.writeLog("获取当前人员groupId:" + groupid + "glrygroupid:" + glrygroupid);
            String value = "";
            if (groupid.equals(glrygroupid)) {
                bb.writeLog("是12小时班制的人");
                Double newValue = divide(String.valueOf(TimeUtil.timeInterval(fromDate + " " + fromTime + ":00", toDate + " " + toTime + ":00")), "3600");
                if (newValue > 0) {
                    value = String.valueOf(newValue);
                } else {
                    value = "0.0";
                }
            } else {
                value = Util.null2String(durationMap.get("duration"));
            }


            bb.writeLog("计算时长timestamp：" + timestamp);
            bb.writeLog("计算时长durationMap：" + JSONObject.toJSONString(durationMap));
            retmap.put("duration", value);
            retmap.put("timestamp", timestamp);
            retmap.put("status", "1");
        } catch (Exception e) {
            retmap.put("status", "-1");
            retmap.put("message", SystemEnv.getHtmlLabelName(382661, user.getLanguage()));
            writeLog(e);
        }
        return retmap;
    }

    private String isRestSerialCalculate(String resourceId, KQWorkTime kqWorkTime) {
        try {
            String fromDate = Util.null2String(params.get("fromDate"));
            String toDate = Util.null2String(params.get("toDate"));
            String fromTime = Util.null2String(params.get("fromTime"));
            String toTime = Util.null2String(params.get("toTime"));
            BigDecimal workTimeTotal = new BigDecimal(TimeUtil.timeInterval(fromDate + " " + fromTime + ":00", toDate + " " + toTime + ":00"));
            BigDecimal restTimeCount = new BigDecimal(0);
            //加班是否跨天
            boolean isAcross = false;
            if (toDate.compareTo(fromDate) > 0) {
                isAcross = true;
            }
            Map<String, Object> serialInfo = kqWorkTime.getSerialInfo(resourceId, fromDate, false);
            String serialId = (String) serialInfo.get(fromDate);
            RecordSet rs = new RecordSet();
            RecordSet rs2 = new RecordSet();
            String querySql = "select rest_shift from kq_ShiftManagement where id=?";
            rs.executeQuery(querySql, serialId);
            if (rs.next()) {
                if (rs.getInt("rest_shift") == 1) {
                    String restStartSql = "select * from kq_ShiftRestTimeSections where serialid=? and resttype='start'";
                    rs.executeQuery(restStartSql, serialId);
                    while (rs.next()) {
                        //休息开始时间
                        String startTime = rs.getString("time");
                        String record1 = rs.getString("record1");
                        //休息结束时间
                        String restEndSql = "select * from kq_ShiftRestTimeSections where record1=? and resttype='end'";
                        rs2.executeQuery(restEndSql, record1);
                        rs2.next();
                        String endTime = rs2.getString("time");
                        if (isAcross) {
                            double restTime1 = TimeRangeCalculator.getRestTime(fromTime, "23:59", startTime, endTime);
                            double restTime2 = TimeRangeCalculator.getRestTime("00:00", toTime, startTime, endTime);
                            restTimeCount = restTimeCount.add(BigDecimal.valueOf(restTime1));
                            restTimeCount = restTimeCount.add(BigDecimal.valueOf(restTime2));

                        } else {
                            double restTime = TimeRangeCalculator.getRestTime(fromTime, toTime, startTime, endTime);
                            restTimeCount = restTimeCount.add(BigDecimal.valueOf(restTime));
                        }
                    }
                }
            }
            Double workTotal = divide(workTimeTotal.subtract(restTimeCount).toString(), "3600");
            return workTotal.toString();
        } catch (Exception e) {
            return null;
        }
    }

    @Override
    public BizLogContext getLogContext() {
        return null;
    }

    /**
     * 提供精确的除法运算。
     *
     * @param v1 被加数
     * @param v2 加数
     * @return 两个参数的和
     */
    public static Double divide(String v1, String v2) {
        BigDecimal b1 = new BigDecimal(v1);
        BigDecimal b2 = new BigDecimal(v2);
        return b1.divide(b2, 2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

}
