package com.engine.nbkq.util;

import cn.hutool.core.io.FileUtil;

import com.engine.nbkq.po.GenerateDocParams;
import weaver.conn.RecordSet;
import weaver.docs.category.SecCategoryComInfo;
import weaver.docs.docs.DocComInfo;
import weaver.docs.docs.DocImageManager;
import weaver.docs.docs.DocManager;
import weaver.docs.docs.DocViewer;
import weaver.file.ImageFileManager;
import weaver.general.BaseBean;
import weaver.general.Util;
import weaver.hrm.User;
import weaver.workflow.request.RequestDoc;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

/**
 * @Author liang.cheng
 * @Date 2023/5/8 5:49 PM
 * @Description: 文件夹 文件操作
 * @Version 1.0
 */
public class FileOperateUtil {


    private static BaseBean bb = new BaseBean();

    /** 删除文件夹
     * @Description:
     * @Author: liang.cheng
     * @Date: 2023/5/8 5:50 PM
     * @param: [folderPath]
     * @return: void
     */
    public static void delFolder(String folderPath) {
        try {
            delAllFile(folderPath); // 删除完里面所有内容
            File myFilePath = new File(folderPath);
            myFilePath.delete(); // 删除空文件夹
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @Description: 删除指定文件夹下所有文件
     * @Author: liang.cheng
     * @Date: 2023/5/8 5:50 PM
     * @param: [path]
     * @return: boolean
     */
    public static boolean delAllFile(String path) {
        boolean flag = false;
        File file = new File(path);
        if (!file.exists()) {
            return flag;
        }
        if (!file.isDirectory()) {
            return flag;
        }
        String[] tempList = file.list();
        File temp = null;
        for (int i = 0; i < tempList.length; i++) {
            if (path.endsWith(File.separator)) {
                temp = new File(path + tempList[i]);
            } else {
                temp = new File(path + File.separator + tempList[i]);
            }
            if (temp.isFile()) {
                temp.delete();
            }
            if (temp.isDirectory()) {
                delAllFile(path + "/" + tempList[i]);// 先删除文件夹里面的文件
                delFolder(path + "/" + tempList[i]);// 再删除空文件夹
                flag = true;
            }
        }
        return flag;
    }

    /**
     * @Description: 生成附件ID
     * @Author: liang.cheng
     * @Date: 2023/5/8 6:02 PM
     * @param: [path, filename]
     * @return: int
     */
    public static int generateImageFileid(String path, String filename) {
        int imagefileid = -1;
        BufferedInputStream inputStream = FileUtil.getInputStream(path);
        try {
            ByteArrayOutputStream bos = null;
            byte[] data = null;
            try {
                byte[] buffer = new byte[1024];
                int len = 0;
                bos = new ByteArrayOutputStream();
                while ((len = inputStream.read(buffer)) != -1) {
                    bos.write(buffer, 0, len);
                }
                data = bos.toByteArray();
                bos.flush();
            } catch (Exception e) {
            } finally {
                if (bos != null) {
                    try {
                        bos.close();
                    } catch (Exception e) {
                    }
                }
                try {
                    inputStream.close();
                } catch (Exception e) {
                }
            }

            if (data == null || data.length == 0) {
                return -2;
            }
            ImageFileManager ifm = new ImageFileManager();
            ifm.setData(data);
            ifm.setImagFileName(filename);
            imagefileid = ifm.saveImageFile();// 文件存储及更新imagefile表
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                inputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return imagefileid;
    }


    /**
     * @Description: 生成文档ID
     * @Author: liang.cheng
     * @Date: 2023/5/26 9:47 AM
     * @param: [user,generateDocParams]
     * @return: int
     */
    public static int generateDocId(User user, GenerateDocParams gdp){

        DocManager dm = new DocManager();
        DocImageManager imgManger = new DocImageManager();
        SecCategoryComInfo scc = new SecCategoryComInfo();
        DocComInfo dc = new DocComInfo();
        DocViewer dv = new DocViewer();

        try {
            RecordSet rs = new RecordSet();
            int docId = dm.getNextDocId(rs);
            String filename = gdp.getFileName();
            imgManger.resetParameter();
            String docSubject = getFileMainName(filename);
            imgManger.setImagefilename(filename);
            imgManger.setDocfiletype(Util.null2String(gdp.getDocType()));
            imgManger.setDocid(docId);
            imgManger.setImagefileid(gdp.getImageFileid());
            imgManger.setIsextfile("1");//1附件
            imgManger.AddDocImageInfo();

            int workflowid= Util.getIntValue(gdp.getWorkflowid(),0);
            String isWorkflowDraft = "0";
            if(workflowid > 0){
                RequestDoc requestDoc=new RequestDoc();
                ArrayList docFiledList = requestDoc.getDocFiled(""+workflowid);
                if(docFiledList != null && docFiledList.size() > 6){
                    isWorkflowDraft = "" +docFiledList.get(6);
                }
            }
            String docStatus = "1"; //生效正常
            if("1".equals(isWorkflowDraft)){
                docStatus = "9"; //流程草稿
            }

            dm.setId(docId);
            dm.setMaincategory(gdp.getMaincategory());
            dm.setSubcategory(gdp.getSubcategory());
            dm.setSeccategory(gdp.getSeccategory());
            dm.setLanguageid(user.getLanguage());
            dm.setDoccontent(Util.null2String(gdp.getDocContent()));
            dm.setDocstatus(docStatus);
            dm.setDocsubject(docSubject);
            dm.setDoccreaterid(user.getUID());
            dm.setDocCreaterType(user.getLogintype());
            dm.setUsertype(user.getLogintype());
            dm.setOwnerid(user.getUID());
            dm.setOwnerType(user.getLogintype());
            dm.setDoclastmoduserid(user.getUID());
            dm.setDocLastModUserType(user.getLogintype());
            dm.setDoccreatedate(gdp.getDocCreateDate());
            dm.setDoclastmoddate(gdp.getDocCreateDate());
            dm.setDoccreatetime(gdp.getDocCreateTime());
            dm.setDoclastmodtime(gdp.getDocCreateTime());
            dm.setDoclangurage(user.getLanguage());
            dm.setKeyword(docSubject);
            dm.setIsapprover("0"); //批复意见
            dm.setIsreply(""); //是否回复文档
            dm.setDocdepartmentid(user.getUserDepartment());
            dm.setDocreplyable("1"); //是否可回复
            dm.setAccessorycount(1); //附件总数
            dm.setParentids("" + docId); //文档父节点
            dm.setOrderable("" + scc.getSecOrderable(gdp.getSeccategory())); //得到此分录下文是否可订阅
            dm.setClientAddress(gdp.getClientAddress());
            dm.setUserid(user.getUID());
            dm.AddDocInfo();
            dm.AddShareInfo();
            dc.addDocInfoCache("" + docId);
            dv.setDocShareByDoc("" + docId);
            return docId;
        }catch (Exception ex) {
            bb.writeLog("文件名为"+gdp.getFileName()+"生成文档失败!");
            return -1;
        }
    }

    /**
     * @Description: 获取文件类型
     * @Author: liang.cheng
     * @Date: 2023/5/26 10:26 AM
     * @param: [docExtendname]
     * @return: int
     */
    public static int docType(String docExtendname){
        int docType = 2;
        switch (docExtendname.toLowerCase()){
            case "doc":
                docType = 3;
                break;
            case "xls":
                docType = 4;
                break;
            case "ppt":
                docType = 5;
                break;
            case "wps":
                docType = 6;
                break;
            case "docx":
                docType = 7;
                break;
            case "xlsx":
                docType = 8;
                break;
            case "pptx":
                docType = 9;
                break;
        }

        return docType;
    }

    /**
     * @Description: 获取文件主名称
     * @Author: liang.cheng
     * @Date: 2023/5/26 11:05 AM
     * @param: [fileName]
     * @return: java.lang.String
     */
    public static String getFileMainName(String fileName) {
        if (fileName == null)
            return "";
        int pos = fileName.lastIndexOf(".");
        if (pos != -1) {
            fileName = fileName.substring(0, pos);
        }
        return fileName;
    }
}
